from typing import Dict

import json
import logging
from urllib.parse import urljoin

import requests


class API:
    def __init__(self, base_url: str, client_token: str):
        self.__client_token = client_token
        self.base_url = base_url

    def authorize(self):
        """ToDo авторизация."""
        pass

    def request(self, url, auth=True, query: Dict = None):
        full_url = urljoin(self.base_url, url)
        headers = {"accept": "application/json, text/plain, */*", "api-version": "3.0"}
        if auth:
            headers["client-token"] = self.__client_token
        resp = requests.get(full_url, headers=headers, params=query)
        try:
            result = resp.json()
            return result
        except Exception as e:
            logging.exception(e)


def try_get(url):
    resp = requests.get(
        url,
        headers={
            "accept": "application/json, text/plain, */*",
            "client-token": "7ba97b6f4049436dab90c789f946ee2f",
            "api-version": "3.0",
        },
    )
    try:
        print(json.dumps(resp.json(), indent=2))
    except:
        print(resp)


def test():
    # GET
    # В ответ пустота (на анонимной сесиии) aggregating_categories: []
    try_get("https://sbermarket.ru/api/stores/83/aggregating_categories")
    # POST -
    # response
    # {"authorize_url":"https://online.sberbank.ru/CSAFront/oidc/sberbank_id/authorize.do",
    # "state":"3dd14bd16f408fb1d326412e7ba41bf813da3f52b5dd2da4",
    # "scope":"openid+name+mobile+email+birthdate+gender+address_of_actual_residence+delivery_address",
    # "client_type":"PRIVATE",
    # "response_type":"code",
    # "client_id":"09a6f842-ed4d-4b42-a925-3bc75e77335e",
    # "nonce":"3345f4d1a1fa345fc44afce6e45eb581a440b0f7e7daf266ba3ae30d042306c5"}
    try_get("https://sbermarket.ru/api/auth_providers/sberbank/auth_params")
    # GET - непонятно пока зачем
    try_get("https://sbermarket.ru/api/multiretailer_order")
    # Получить свои текущие адреса.
    # запрос без токена, но в куках есть remember_user_token -ни на что не похожий
    # в ответ на
    try_get("https://sbermarket.ru/api/addresses")
    # Отправляется без токена а в ответа получаем
    # {"csrf_param":"authenticity_token","csrf_token":"Jbdkmim0THL6WdLkI5+12mCFI1mMrWDMQIu2OzLEJwOG7Qh6PJSH3HYTA3om9n1yk1YQh4FC/BOehA6K2tZMcg=="}
    # В ответ также получаем cookie: _Instamart_session
    try_get("https://sbermarket.ru/api/next/page/browser_head")
    # POST phone: "tfRoupxZfXaStCfaprZh75Le1bbzxGMa3Wv9Zxt2PFo=" - динамический каждый раз понять из js не получилось как воссоздать это
    #
    try_get("https://sbermarket.ru/api/phone_confirmations")

    try_get("https://sbermarket.ru/api/stores/1/next_deliveries?cargo=false")
    # PUT method, положить адрес
    # {
    #     "validity": 0,
    #     "lat": 59.962246,
    #     "lon": 30.319758,
    #     "street": "Большая Монетная улица",
    #     "building": "16",
    #     "full_address": "Большая Монетная улица, 16",
    #     "city": "Санкт-Петербург",
    #     "city_in": ""
    # }
    try_get("https://sbermarket.ru/api/shopping_context")
    # Последние магазины на основе адреса?
    # response:
    # {"pickup":[],"by_courier":[]}
    try_get("https://sbermarket.ru/api/recent_stores?lat=59.962246&lon=30.319758")
    # Это возвращает client-token вида: {"api-version":"3.0","client-token":"TOKEN","is-storefront-ssr":l.sk}
    try_get("https://sbermarket.ru/_next/static/chunks/pages/_app-b824ce51c817753c.js")



from sbermarket_api.api import API


class Product:
    """{
    'id': '482975',
    'legacy_offer_id': 268013250,
    'legacy_product_id': 66942,
    'sku': '60020', # Иногда этот параметр может совпадать между товарами.
    'retailer_sku': '482975',
    'name': 'Сок J7 яблоко 970 мл',
    'price': 159.01,
    'original_price': 159.01,
    'discount': 0.0,
    'human_volume': '970 мл.',
    'volume': 970.0,
    'volume_type': 'ml',
    'items_per_pack': 1,
    'discount_ends_at': None,
    'price_type': 'per_item',
    'grams_per_unit': 970,
    'unit_price': 159.01,
    'original_unit_price': 159.01,
    'slug': 'sok-j7-iabloko-4',
    'max_select_quantity': 999,
    'canonical_url': 'https://sbermarket.ru/products/60020-sok-j7-iabloko-4',
    'available': True,
    'vat_info': None,
    'bmpl_info': None,
    'promo_badge_ids': [],
    'requirements': [],
    'image_urls': ['https://sbermarket.ru/spree/products/3139001/original/60020_2.jpg?1636780049',
                    'https://sbermarket.ru/spree/products/3139000/original/60020.jpg?1636780048']
                }
    """

    def __init__(self, api: API, name: str = "", sku: str = "", **kwargs):
        self.api = api
        self.sku = sku
        self.name = name
        self.__dict__.update(kwargs)

    def __str__(self):
        return f"{self.name} #{self.sku}"



from sbermarket_api.api import API
from sbermarket_api.product import Product


class Store:
    def __init__(self, api: API, store_id: int, name: str, **kwargs):
        self.api = api
        self.store_id = store_id
        self.name = name
        self.__dict__.update(kwargs)

    def search_products(self, search: str, per_page: int = 24, page: int = 1):
        """Поиск по товарам per_page 24 это максимум"""
        query = {"q": search, "per_page": per_page, "page": page}
        response = self.api.request(f"stores/{self.store_id}/products", query=query)
        return [Product(self.api, **product) for product in response["products"]]

    def categories(self, depth: int = 2):
        """Получить инфу по категориям, товаров там не будет"""
        query = {}
        if depth:
            query["depth"] = depth
        return self.api.request(f"stores/{self.store_id}/categories", query=query)

    def product(self, id: str):
        """Получить инфу по одному товару"""
        url = f"stores/{self.store_id}/products/{id}"
        return Product(self.api, **self.api.request(url))

    def __str__(self):
        return self.name



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['sbermarket_api']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.28.1,<3.0.0']

setup_kwargs = {
    'name': 'sbermarket-api',
    'version': '0.0.1',
    'description': 'Not official python client for product market sbermarket API.',
    'long_description': '# sbermarket-api\n\n<div align="center">\n\n[![Build status](https://github.com/nov1kov/sbermarket-api/workflows/build/badge.svg?branch=master&event=push)](https://github.com/nov1kov/sbermarket-api/actions?query=workflow%3Abuild)\n[![Python Version](https://img.shields.io/pypi/pyversions/sbermarket-api.svg)](https://pypi.org/project/sbermarket-api/)\n[![Dependencies Status](https://img.shields.io/badge/dependencies-up%20to%20date-brightgreen.svg)](https://github.com/nov1kov/sbermarket-api/pulls?utf8=%E2%9C%93&q=is%3Apr%20author%3Aapp%2Fdependabot)\n\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n[![Security: bandit](https://img.shields.io/badge/security-bandit-green.svg)](https://github.com/PyCQA/bandit)\n[![Pre-commit](https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white)](https://github.com/nov1kov/sbermarket-api/blob/master/.pre-commit-config.yaml)\n[![Semantic Versions](https://img.shields.io/badge/%20%20%F0%9F%93%A6%F0%9F%9A%80-semantic--versions-e10079.svg)](https://github.com/nov1kov/sbermarket-api/releases)\n[![License](https://img.shields.io/github/license/nov1kov/sbermarket-api)](https://github.com/nov1kov/sbermarket-api/blob/master/LICENSE)\n![Coverage Report](assets/images/coverage.svg)\n\nNot official python client for product market Sbermarket.\n\n</div>\n\n## Задачи что преследовал:\n\n1. Поиск самого дешевого товара среди магазинов вокруг.\n2. Отлеживание в целом цен на продукты.\n3. Сделать алерт на дешевые товары.\n4. Вести статистику на цены товаров.\n\n## Что внутри?\n\nТокен получается только из статического js скрипта. Где содержится JSON вида:\n```json\n{"api-version":"3.0","client-token":"TOKEN","is-storefront-ssr":l.sk}\n```\nСкорее всего он у всех одинаковый.\n\n## Installation\n\n```bash\npip install -U sbermarket-api\n```\n\nor install with `Poetry`\n\n```bash\npoetry add sbermarket-api\n```\n\n## 📈 Releases\n\nYou can see the list of available releases on the [GitHub Releases](https://github.com/nov1kov/sbermarket-api/releases) page.\n\nWe follow [Semantic Versions](https://semver.org/) specification.\n\nWe use [`Release Drafter`](https://github.com/marketplace/actions/release-drafter). As pull requests are merged, a draft release is kept up-to-date listing the changes, ready to publish when you’re ready. With the categories option, you can categorize pull requests in release notes using labels.\n\n### List of labels and corresponding titles\n\n|               **Label**               |  **Title in Releases**  |\n| :-----------------------------------: | :---------------------: |\n|       `enhancement`, `feature`        |       🚀 Features       |\n| `bug`, `refactoring`, `bugfix`, `fix` | 🔧 Fixes & Refactoring  |\n|       `build`, `ci`, `testing`        | 📦 Build System & CI/CD |\n|              `breaking`               |   💥 Breaking Changes   |\n|            `documentation`            |    📝 Documentation     |\n|            `dependencies`             | ⬆️ Dependencies updates |\n\nYou can update it in [`release-drafter.yml`](https://github.com/nov1kov/sbermarket-api/blob/master/.github/release-drafter.yml).\n\nGitHub creates the `bug`, `enhancement`, and `documentation` labels for you. Dependabot creates the `dependencies` label. Create the remaining labels on the Issues tab of your GitHub repository, when you need them.\n\n## 🛡 License\n\n[![License](https://img.shields.io/github/license/nov1kov/sbermarket-api)](https://github.com/nov1kov/sbermarket-api/blob/master/LICENSE)\n\nThis project is licensed under the terms of the `MIT` license. See [LICENSE](https://github.com/nov1kov/sbermarket-api/blob/master/LICENSE) for more details.\n\n## 📃 Citation\n\n```bibtex\n@misc{sbermarket-api,\n  author = {Nov1kov},\n  title = {Not official python client for product market sbermarket API.},\n  year = {2022},\n  publisher = {GitHub},\n  journal = {GitHub repository},\n  howpublished = {\\url{https://github.com/nov1kov/sbermarket-api}}\n}\n```\n\n## Credits [![🚀 Your next Python package needs a bleeding-edge project structure.](https://img.shields.io/badge/python--package--template-%F0%9F%9A%80-brightgreen)](https://github.com/TezRomacH/python-package-template)\n\nThis project was generated with [`python-package-template`](https://github.com/TezRomacH/python-package-template)\n\n## Alternatives\n\n1. Sbermarket client on TypeScript https://github.com/x0rium/sbermarket-api\n',
    'author': 'Nov1kov',
    'author_email': 'spellh1@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/nov1kov/sbermarket-api',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from typing import List

from sbermarket_api.api import API
from sbermarket_api.store import Store

DEFAULT_TOKEN = "7ba97b6f4049436dab90c789f946ee2f"


class Client:
    def __init__(self, base_url="https://sbermarket.ru/api/", client_token: str = DEFAULT_TOKEN):
        self.api = API(base_url=base_url, client_token=client_token)

    def stores(self, lat: float, lon: float, shipping_method="delivery") -> List[Store]:
        """lat, lon - обязательные
        если указать shipping_method = None - выдаст 1000+ точек. Возможно все в России?
        возможные параметры еще: include=closest_shipping_options,labels,retailer,label_store_ids"""
        query = {"lat": lat, "lon": lon}
        if shipping_method:
            query["shipping_method"] = shipping_method
        return [Store(self.api, **store) for store in self.api.request("stores", query=query)]

    def store(self, id: int) -> Store:
        """Получить полную информацию по магазину"""
        result = self.api.request(f"stores/{id}")["store"]
        return Store(self.api, store_id=result["id"], **result)

    def shopping_session(self):
        """получить available_stores, favorite_product_skus, store_labels,
        В ответ если это не анонимная сессия может придти полная инфа о пользователе."""
        return self.api.request("shopping_session", auth=False)
